import re

from django.conf import settings
from django.conf.urls import patterns, url


def assetfiles_urlpatterns(prefix=None, insecure=False, **kwargs):
    """
    Helper function to return a URL pattern for serving asset files.
    """
    if prefix is None:
        prefix = settings.STATIC_URL
    if (not settings.DEBUG and not insecure) or ('://' in prefix):
        return []

    kwargs['insecure'] = insecure
    return patterns('',
        url(
            r'^%s(?P<path>.*)$' % re.escape(prefix.lstrip('/')),
            'assetfiles.views.serve',
            kwargs=kwargs
        ),
    )



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from assetfiles.tests.base import AssetfilesTestCase


class TestServe(AssetfilesTestCase):
    def test_returns_not_found_without_an_asset(self):
        response = self.client.get('/static/non/existent/file.css')
        self.assertEqual(response.status_code, 404)

    def test_returns_static_files(self):
        self.mkfile('static/css/static.css', 'body { color: red; }')
        response = self.client.get('/static/css/static.css')
        self.assertContains(response, 'body { color: red; }')

    def test_returns_static_files_with_correct_content_type(self):
        self.mkfile('static/css/static.css')
        response = self.client.get('/static/css/static.css')
        self.assertEqual(response.get('content-type'), 'text/css')

    def test_returns_static_files_with_extra_extensions(self):
        self.mkfile('app-1/static/js/jquery.plugin.js', '$.fn.plugin = {};')
        response = self.client.get('/static/js/jquery.plugin.js')
        self.assertContains(response, '$.fn.plugin = {};')

    def test_returns_app_static_files(self):
        self.mkfile('app-1/static/css/app_static.css', 'body { color: blue; }')
        response = self.client.get('/static/css/app_static.css')
        self.assertContains(response, 'body { color: blue; }')

    def test_processes_scss_files(self):
        self.mkfile('static/css/simple.scss',
            '$c: red; body { color: $c; }')
        response = self.client.get('/static/css/simple.css')
        self.assertContains(response, 'body {\n  color: red; }')

    def test_returns_processed_scss_files_with_correct_content_type(self):
        self.mkfile('static/css/simple.scss',
            '$c: red; body { color: $c; }')
        response = self.client.get('/static/css/simple.css')
        self.assertEqual(response.get('content-type'), 'text/css')

    def test_processes_app_scss_files(self):
        self.mkfile('app-1/static/css/app.scss',
            '$c: yellow; body { color: $c; }')
        response = self.client.get('/static/css/app.css')
        self.assertContains(response, 'body {\n  color: yellow; }')

    def test_processes_scss_files_with_deps(self):
        self.mkfile('static/css/folder/_dep.scss', '$c: black;')
        self.mkfile('static/css/with_deps.scss',
            '@import "folder/dep"; body { color: $c; }')
        response = self.client.get('/static/css/with_deps.css')
        self.assertContains(response, 'body {\n  color: black; }')

    def test_processes_scss_files_with_app_deps(self):
        self.mkfile('app-1/static/css/folder/_dep.scss', '$c: white;')
        self.mkfile('static/css/with_app_deps.scss',
            '@import "folder/dep"; body { color: $c; }')
        response = self.client.get('/static/css/with_app_deps.css')
        self.assertContains(response, 'body {\n  color: white; }')

    def test_processes_asset_files_with_unicode_chars(self):
        self.mkfile('static/css/simple.scss',
            '$c: "é"; a::before { content: $c; }')
        self.mkfile('static/js/simple.coffee', 'a = foo: "é#{2}3"')
        response = self.client.get('/static/css/simple.css')
        self.assertContains(response, '@charset "UTF-8";')
        self.assertContains(response, 'a::before {\n  content: "é"; }')
        response = self.client.get('/static/js/simple.js')
        self.assertContains(response, 'foo: "é" + 2 + "3"')



from django.core.exceptions import ImproperlyConfigured
from django.utils.datastructures import SortedDict
from django.utils.functional import memoize
from django.utils.importlib import import_module

from assetfiles import settings
from assetfiles.filters.base import BaseFilter
from assetfiles.filters.mixins import MultiInputMixin, ExtensionMixin


def find_by_input_path(input_path):
    """
    Returns the first filter that would accept the given path as input.

    Args:
        input_path: An absolute path to the file
    Returns:
        The found filter instance or `None`
    """
    for filter in get_filters():
        if filter.matches_input(input_path):
            return filter
    return None


def find_by_output_path(output_path):
    """
    Returns the first filter that would output to the given path.

    Args:
        output_path: An absolute path to the file
    Returns:
        The found filter instance or `None`
    """
    for filter in get_filters():
        if filter.matches_output(output_path):
            return filter
    return None


def get_filters():
    """
    Returns filter instances configured in ASSETFILES_FILTERS.
    """
    for filter_path in settings.FILTERS:
        yield get_filter(filter_path)


def _get_filter(import_path):
    """
    Imports the assetfiles filter class described by import_path, where
    import_path is the full Python path to the class.
    """
    module, attr = import_path.rsplit('.', 1)
    try:
        mod = import_module(module)
    except ImportError as e:
        raise ImproperlyConfigured('Error importing module %s: "%s"' %
                                   (module, e))
    try:
        Filter = getattr(mod, attr)
    except AttributeError:
        raise ImproperlyConfigured('Module "%s" does not define a "%s" '
                                   'class.' % (module, attr))
    if not issubclass(Filter, BaseFilter):
        raise ImproperlyConfigured('Filter "%s" is not a subclass of "%s"' %
                                   (Filter, BaseFilter))
    return Filter()

_filters = SortedDict()
get_filter = memoize(_get_filter, _filters, 1)



from __future__ import unicode_literals

from assetfiles.filters.coffee import CoffeeScriptFilterError
from assetfiles.tests.base import assertRaisesRegex, AssetfilesTestCase, filter


class TestCoffeeScriptFilter(AssetfilesTestCase):
    def test_processes_coffee_files(self):
        self.mkfile('static/js/simple.coffee', 'a = foo: "1#{2}3"')
        self.assertIn(b'foo: "1" + 2 + "3"', filter('js/simple.js'))

    def test_raises_syntax_error(self):
        with assertRaisesRegex(
                self,
                CoffeeScriptFilterError,
                r'.*?SyntaxError.*?static/js/simple\.coffee.*?line 5'):
            self.mkfile('static/js/simple.coffee', '\n\n\n\na = foo: "1#{2}3')
            filter('js/simple.js')



from optparse import make_option

from django.conf import settings
from django.core.management.commands.runserver import Command as RunserverCommand

from assetfiles.handlers import AssetFilesHandler


class Command(RunserverCommand):
    """
    Overrides the default `runserver` command to use assetfiles' handler.
    """

    option_list = RunserverCommand.option_list + (
        make_option('--nostatic', action='store_false', dest='use_static_handler', default=True,
            help='Tells Django to NOT automatically serve static files at STATIC_URL.'),
        make_option('--insecure', action='store_true', dest='insecure_serving', default=False,
            help='Allows serving static files even if DEBUG is False.'),
    )
    help = 'Starts a lightweight Web server for development and also serves static files.'

    def get_handler(self, *args, **options):
        handler = super(Command, self).get_handler(*args, **options)
        use_static_handler = options.get('use_static_handler', True)
        insecure_serving = options.get('insecure_serving', False)
        if use_static_handler and (settings.DEBUG or insecure_serving):
            return AssetFilesHandler(handler)
        return handler



from os import path
import shutil


TESTS_ROOT = path.abspath(path.dirname(__file__))
PROJECT_ROOT = path.join(TESTS_ROOT, 'project')

# We'll use am empty project for each test and create the files we need.
# But, we need actual app directories to have them installed so we copy
# the template beforehand.
if not path.exists(PROJECT_ROOT):
    shutil.copytree(path.join(TESTS_ROOT, 'project-template'), PROJECT_ROOT)

TEST_RUNNER = 'django_nose.run_tests'

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ':memory:',
    }
}

INSTALLED_APPS = (
    'django_nose',
    'assetfiles',
    'assetfiles.tests.project.app-1',
    'assetfiles.tests.project.app-2',
)

ROOT_URLCONF = 'assetfiles.tests.urls'

STATIC_ROOT = path.join(PROJECT_ROOT, 'public')

STATIC_URL = '/static/'

STATICFILES_DIRS = (
    path.join(PROJECT_ROOT, 'static'),
    ('prefix', path.join(PROJECT_ROOT, 'static-prefix'))
)

STATICFILES_FINDERS = (
    'django.contrib.staticfiles.finders.FileSystemFinder',
    'django.contrib.staticfiles.finders.AppDirectoriesFinder',
)

TEMPLATE_DIRS = (
    path.join(PROJECT_ROOT, 'templates'),
)

SECRET_KEY = 'ev2pj15ucf^d84l216^@-mv)pl4$^@9g4)9_)7xi@0j0xop94f'
DEFAULT_CHARSET = 'utf-8'



import re

from django.utils import six


class GlobInputMixin(object):
    def __init__(self, input_path_glob, **kwargs):
        self.input_path_glob = input_path_glob

    def matches_input(self, input_path):
        from fnmatch import fnmatch

        return fnmatch(input_path, self.input_path_glob)

    def derive_input_paths(self, output_path):
        input_paths = []
        for root in self._get_staticfiles_roots():
            for path in self._iglob_within(root, self.input_path_glob):
                if path not in input_paths:
                    input_paths.append(path)
        return input_paths

    def _iglob_within(self, root, pathname):
        try:
            from glob2 import iglob
        except ImportError:
            from glob import iglob
        import os

        for path in iglob(os.path.join(root, pathname)):
            yield path.replace(root, '').lstrip('/')

    def _get_staticfiles_roots(self):
        for storages in self._get_staticfiles_storages():
            for root, storage in six.iteritems(storages):
                if hasattr(storage, 'location'):
                    yield storage.location

    def _get_staticfiles_storages(self):
        from django.contrib.staticfiles.finders import get_finders

        for finder in get_finders():
            if hasattr(finder, 'storages'):
                yield finder.storages


class ListInputMixin(object):
    def __init__(self, input_paths):
        self.input_paths = input_paths

    def matches_input(self, input_path):
        return input_path in self.input_paths

    def derive_input_paths(self, output_path):
        return list(self.input_paths)


class MultiInputMixin(object):
    def __init__(self, input_paths=None, *args, **kwargs):
        super(MultiInputMixin, self).__init__(*args, **kwargs)
        if isinstance(input_paths, str):
            self._multi_input_delegate = GlobInputMixin(input_path_glob=input_paths)
        else:
            self._multi_input_delegate = ListInputMixin(input_paths=input_paths)

    def _matches_input(self, input_path):
        return self._multi_input_delegate.matches_input(input_path)

    def _derive_input_paths(self, output_path):
        return self._multi_input_delegate.derive_input_paths(output_path)


class ExtensionMixin(object):
    """
    A mixin for filters that take a file with a certain file extension and
    output another.

    For example, SassFilter generally takes files with the exentions ".scss" and
    ".sass" and output CSS files (".css").

    Attributes:
        input_exts: A tuple of extensions (without the prefixed ".")
        output_ext: A single output extension (again, without the prefixed ".")
    """
    input_exts = None
    output_ext = None

    def __init__(self, input_ext=None, input_exts=None, output_ext=None, *args, **kwargs):
        super(ExtensionMixin, self).__init__(*args, **kwargs)
        if input_ext:
            self.input_ext = input_ext
        if input_exts:
            self.input_exts = input_exts
        if output_ext:
            self.output_ext = output_ext
        if not self.input_exts and self.input_ext:
            self.input_exts = (self.input_ext,)

    def _matches_input(self, intput_path):
        if self.input_exts:
            return re.search(r'\.({0})$'.format('|'.join(self.input_exts)), intput_path)
        return False

    def _matches_output(self, output_path):
        if self.output_ext:
            return re.search(r'\.{0}$'.format(self.output_ext), output_path)
        return False

    def _derive_input_paths(self, output_path):
        paths = []
        if self.input_exts:
            for ext in self.input_exts:
                ext = '.' + ext
                paths.append(output_path + ext)
                paths.append(re.sub(r'\.[^\.]*$', ext, output_path))
        return paths

    def _derive_output_path(self, input_path):
        if self.output_ext:
            path = re.sub(r'\.{0}'.format(self.output_ext), '', input_path)
            ext = '.' + self.output_ext
            return re.sub(r'\.[^\.]*$', ext, path)
        return None

    def set_input_ext(self, value):
        self.input_exts = (value,)
    input_ext = property(fset=set_input_ext)



from __future__ import unicode_literals

import re

from assetfiles.filters.sass import SassFilterError
from assetfiles.tests.base import assertRaisesRegex, AssetfilesTestCase, filter


class TestSassFilter(AssetfilesTestCase):
    def test_processes_scss_files(self):
        self.mkfile(
            'static/css/simple.scss',
            '$c: red; body { color: $c; }')
        self.assertEqual(filter('css/simple.css'), b'body {\n  color: red; }')

    def test_processes_app_scss_files(self):
        self.mkfile(
            'app-1/static/css/app.scss',
            '$c: yellow; body { color: $c; }')
        self.assertEqual(filter('css/app.css'), b'body {\n  color: yellow; }')

    def test_processes_scss_files_with_deps(self):
        self.mkfile('static/css/folder/_dep.scss', '$c: black;')
        self.mkfile(
            'static/css/with_deps.scss',
            '@import "folder/dep"; body { color: $c; }')
        self.assertEqual(
            filter('css/with_deps.css'),
            b'body {\n  color: black; }')

    def test_processes_scss_files_with_app_deps(self):
        self.mkfile('app-1/static/css/folder/_dep.scss', '$c: white;')
        self.mkfile(
            'static/css/with_app_deps.scss',
            '@import "folder/dep"; body { color: $c; }')
        self.assertEqual(
            filter('css/with_app_deps.css'),
            b'body {\n  color: white; }')

    def test_integrates_static_url_with_sass(self):
        self.mkfile(
            'static/css/with_url.scss',
            'body { background: static-url("img/bg.jpg"); }')
        self.assertEqual(
            filter('css/with_url.css'),
            b'body {\n  background: url("/static/img/bg.jpg"); }')

    def test_integrates_with_compass(self):
        self.mkfile(
            'static/css/with_compass.scss',
            '@import "compass"; .btn { @include border-radius(5px); }')
        self.assertIn(b'border-radius: 5px;', filter('css/with_compass.css'))

    def test_integrates_with_compass_image_url(self):
        self.mkfile(
            'static/css/image_url.scss',
            'body { background: image-url("img/bg.jpg"); }')
        self.mkfile(
            'static/css/only_path.scss',
            'body { background: url(image-url("img/bg.jpg", true)); }')
        self.mkfile(
            'static/css/cache_buster.scss',
            'body { background: image-url("img/bg.jpg", false, true); }')
        self.assertEqual(
            filter('css/image_url.css'),
            b'body {\n  background: url("/static/img/bg.jpg"); }')
        self.assertEqual(
            filter('css/only_path.css'),
            b'body {\n  background: url("/static/img/bg.jpg"); }')
        self.assertEqual(
            filter('css/cache_buster.css'),
            b'body {\n  background: url("/static/img/bg.jpg"); }')

    def test_integrates_with_compass_font_url(self):
        self.mkfile(
            'static/css/font_url.scss',
            '@font-face { src: font-url("fonts/font.ttf"); }')
        self.mkfile(
            'static/css/only_path.scss',
            '@font-face { src: url(font-url("fonts/font.ttf", true)); }')
        self.assertEqual(
            filter('css/font_url.css'),
            b'@font-face {\n  src: url("/static/fonts/font.ttf"); }')
        self.assertEqual(
            filter('css/only_path.css'),
            b'@font-face {\n  src: url("/static/fonts/font.ttf"); }')

    def test_raises_syntax_error(self):
        message = re.compile(
            r'.*?Syntax error.*?line 5.*?static/css/syntax_error\.scss',
            re.DOTALL)
        with assertRaisesRegex(self, SassFilterError, message):
            self.mkfile('static/css/syntax_error.scss', '\n\n\n\nbody {')
            filter('css/syntax_error.css')



from django.conf import settings
from django.core.files.base import File, ContentFile
from django.core.files.storage import Storage
from django.utils.encoding import filepath_to_uri
from django.utils import six


class TempFilesStorage(Storage):
    """
    TempFilesStorage is an in-memory storage of files and file-like strings.

    This is used to temporarily hold processed files before they're copied
    to a new storage during `collectstatic`
    """

    def __init__(self):
        self.files = {}

    def _open(self, name, mode='rb'):
        if not self.exists(name):
            raise IOError('No such file in TempFilesStorage: {0}'.format(name))
        file = self.files[name]

        if isinstance(file, six.string_types) or isinstance(file, six.binary_type):
            file = ContentFile(file, name)
        elif not isinstance(file, File):
            file = File(file)

        return file

    def _save(self, name, content):
        self.files[name] = content
        return name

    def delete(self, name):
        del self.files[name]

    def exists(self, name):
        return name in self.files



import unittest

from assetfiles.filters import BaseFilter, MultiInputMixin, ExtensionMixin
from assetfiles.tests.base import is_glob2_available, AssetfilesTestCase


class MultiInputFilter(MultiInputMixin, BaseFilter):
    pass


class TestMultiInputMixin(AssetfilesTestCase):
    def mk_project_files(self):
        self.mkfile('static/file1.in')
        self.mkfile('static/dir1/file2.in')
        self.mkfile('static/dir1/file3.out')
        self.mkfile('static/dir2/file4.in')
        self.mkfile('app-1/static/file5.out')
        self.mkfile('app-1/static/file6.in')
        self.mkfile('app-1/static/dir1/file7.in')
        self.mkfile('app-1/static/dir3/file8.in')
        self.mkfile('app-2/static/file10.in')
        self.mkfile('app-2/static/dir4/subdir/file9.in')

    def test_matches_input_set_with_list(self):
        filter = MultiInputFilter(input_paths=(
            'dir/file1.in',
            'dir/file2.in',
            'dir/file3.in',
        ))
        self.assertFalse(filter.matches_input('file1.in'))
        self.assertFalse(filter.matches_input('dir/file1.out'))
        self.assertFalse(filter.matches_input('dir/file4.in'))
        self.assertFalse(filter.matches_input('dir/dir/file1.in'))
        self.assertTrue(filter.matches_input('dir/file1.in'))
        self.assertTrue(filter.matches_input('dir/file2.in'))
        self.assertTrue(filter.matches_input('dir/file3.in'))

    def test_matches_input_set_with_glob(self):
        filter = MultiInputFilter(input_paths='*.in')
        self.assertTrue(filter.matches_input('file1.in'))
        self.assertFalse(filter.matches_input('file1.out'))
        self.assertFalse(filter.matches_input('dir/file1.out'))
        self.assertTrue(filter.matches_input('dir/file4.in'))
        self.assertTrue(filter.matches_input('dir/dir/file1.in'))
        self.assertTrue(filter.matches_input('dir/file1.in'))
        self.assertTrue(filter.matches_input('dir/file2.in'))
        self.assertTrue(filter.matches_input('dir/file3.in'))

    def test_derives_input_paths_set_with_list(self):
        filter = MultiInputFilter(input_paths=(
            'dir/file1.in',
            'dir/file2.in',
            'dir/file3.in',
        ))
        self.assertEqual(filter.derive_input_paths('dir/file1.out'), [
            'dir/file1.in',
            'dir/file2.in',
            'dir/file3.in',
        ])

    def test_derives_input_paths_set_with_glob(self):
        filter1 = MultiInputFilter(input_paths='*.in')
        filter2 = MultiInputFilter(input_paths='dir1/*.in')
        self.mk_project_files()
        self.assertEqual(filter1.derive_input_paths('file.out'), [
            'file1.in',
            'file6.in',
            'file10.in',
        ])
        self.assertEqual(filter2.derive_input_paths('file.out'), [
            'dir1/file2.in',
            'dir1/file7.in',
        ])

    @unittest.skipUnless(is_glob2_available(), 'glob2 is not Python 3 compatible')
    def test_derives_input_paths_set_with_glob2(self):
        filter = MultiInputFilter(input_paths='**/*.in')
        self.mk_project_files()
        self.assertEqual(filter.derive_input_paths('file.out'), [
            'file1.in',
            'dir1/file2.in',
            'dir2/file4.in',
            'file6.in',
            'dir1/file7.in',
            'dir3/file8.in',
            'file10.in',
            'dir4/subdir/file9.in',
        ])

    def test_matches_set_input_path(self):
        filter = MultiInputFilter(input_path='dir/main.in')
        self.assertFalse(filter.matches_input('main.in'))
        self.assertFalse(filter.matches_input('dir/main.out'))
        self.assertFalse(filter.matches_input('dir/main'))
        self.assertFalse(filter.matches_input('dir/dir/main.in'))
        self.assertTrue(filter.matches_input('dir/main.in'))

    def test_derives_set_input_path(self):
        filter = MultiInputFilter(input_path='dir/main.in')
        self.assertEqual(filter.derive_input_paths('dir/main.in'),
            ['dir/main.in'])


class ExtensionFilter(ExtensionMixin, BaseFilter):
    input_exts = ('foo', 'baz')
    output_ext = 'bar'


class TessExtensionMixin(AssetfilesTestCase):
    def test_matches_input_file_by_ext(self):
        filter = ExtensionFilter()
        self.assertTrue(filter.matches_input('main.foo'))
        self.assertTrue(filter.matches_input('main.baz'))
        self.assertTrue(filter.matches_input('main.plugin.foo'))
        self.assertFalse(filter.matches_input('main.css'))

    def test_does_not_match_input_file_without_input_exts(self):
        filter = ExtensionFilter()
        filter.input_exts = ()
        self.assertFalse(filter.matches_input('main.foo'))
        self.assertFalse(filter.matches_input('main.baz'))

    def test_set_single_input_ext(self):
        filter = ExtensionFilter()
        filter.input_ext = 'foo'
        self.assertEqual(('foo',), filter.input_exts)

    def test_matches_output_file_by_ext(self):
        filter = ExtensionFilter()
        self.assertTrue(filter.matches_output('main.bar'))
        self.assertTrue(filter.matches_output('main.plugin.bar'))
        self.assertFalse(filter.matches_output('main.foo'))

    def test_does_not_match_output_file_without_output_ext(self):
        filter = ExtensionFilter()
        filter.output_ext = None
        self.assertFalse(filter.matches_output('main.bar'))

    def test_derive_input_paths(self):
        filter = ExtensionFilter()
        self.assertEqual([
            'dir/main.bar.foo',
            'dir/main.foo',
            'dir/main.bar.baz',
            'dir/main.baz',
        ], filter.derive_input_paths('dir/main.bar'))
        self.assertEqual([
            'dir/main.plugin.bar.foo',
            'dir/main.plugin.foo',
            'dir/main.plugin.bar.baz',
            'dir/main.plugin.baz',
        ], filter.derive_input_paths('dir/main.plugin.bar'))

    def test_derive_output_path(self):
        filter = ExtensionFilter()
        self.assertEqual('dir/main.bar',
            filter.derive_output_path('dir/main.foo'))
        self.assertEqual('dir/main.bar',
            filter.derive_output_path('dir/main.bar.foo'))
        self.assertEqual('dir/main.plugin.bar',
            filter.derive_output_path('dir/main.plugin.foo'))

    def test_matches_set_input_path(self):
        filter = ExtensionFilter(input_path='dir/main.in')
        self.assertFalse(filter.matches_input('main.in'))
        self.assertFalse(filter.matches_input('dir/main.out'))
        self.assertFalse(filter.matches_input('dir/main'))
        self.assertFalse(filter.matches_input('dir/dir/main.in'))
        self.assertTrue(filter.matches_input('dir/main.in'))

    def test_derives_set_input_path(self):
        filter = ExtensionFilter(input_path='dir/main.in')
        self.assertEqual(filter.derive_input_paths('dir/main.in'),
            ['dir/main.in'])

    def test_matches_set_output_path(self):
        filter = ExtensionFilter(output_path='dir/main.out')
        self.assertFalse(filter.matches_output('main.out'))
        self.assertFalse(filter.matches_output('dir/main.in'))
        self.assertFalse(filter.matches_output('dir/main'))
        self.assertFalse(filter.matches_output('dir/dir/main.out'))
        self.assertTrue(filter.matches_output('dir/main.out'))

    def test_derives_set_output_path(self):
        filter = ExtensionFilter(output_path='dir/main.out')
        self.assertEqual(filter.derive_output_path('main.in'), 'dir/main.out')
        self.assertEqual(filter.derive_output_path('dir/main.in'), 'dir/main.out')



from assetfiles import filters, settings
from assetfiles.filters import BaseFilter, ExtensionMixin
from assetfiles.tests.base import AssetfilesTestCase


class Filter1(ExtensionMixin, BaseFilter):
    input_exts = ('in', 'in1')
    output_ext = 'out'


class Filter2(ExtensionMixin, BaseFilter):
    input_ext = 'in2'
    output_ext = 'out2'


class TestFilters(AssetfilesTestCase):
    def setUp(self):
        super(TestFilters, self).setUp()
        self.old_filters = settings.FILTERS
        settings.FILTERS = (
            'assetfiles.tests.filters.test_base.Filter1',
            'assetfiles.tests.filters.test_base.Filter2',
        )

    def tearDown(self):
        settings.FILTERS = self.old_filters

    def test_find_by_input_path(self):
        self.assertIsInstance(filters.find_by_input_path('main.in'), Filter1)
        self.assertIsInstance(filters.find_by_input_path('main.in1'), Filter1)
        self.assertIsInstance(filters.find_by_input_path('main.in2'), Filter2)
        self.assertEqual(None, filters.find_by_input_path('main.out'))

    def test_find_by_output_path(self):
        self.assertIsInstance(filters.find_by_output_path('main.out'), Filter1)
        self.assertIsInstance(filters.find_by_output_path('main.out2'), Filter2)
        self.assertEqual(None, filters.find_by_output_path('main.in'))


class ReplaceFilter(BaseFilter):
    def __init__(self, pattern, replacement, **kwargs):
        super(ReplaceFilter, self).__init__(**kwargs)
        self.pattern = pattern
        self.replacement = replacement

    def filter(self, input):
        with open(input, 'r') as file:
            return file.read().replace(self.pattern, self.replacement)


class TestBaseFilter(AssetfilesTestCase):
    def test_matches_set_input_path(self):
        filter = BaseFilter(input_path='dir/main.in')
        self.assertFalse(filter.matches_input('main.in'))
        self.assertFalse(filter.matches_input('dir/main.out'))
        self.assertFalse(filter.matches_input('dir/main'))
        self.assertFalse(filter.matches_input('dir/dir/main.in'))
        self.assertTrue(filter.matches_input('dir/main.in'))

    def test_derives_set_input_path(self):
        filter = BaseFilter(input_path='dir/main.in')
        self.assertEqual(filter.derive_input_paths('dir/main.in'),
            ['dir/main.in'])

    def test_matches_set_output_path(self):
        filter = BaseFilter(output_path='dir/main.out')
        self.assertFalse(filter.matches_output('main.out'))
        self.assertFalse(filter.matches_output('dir/main.in'))
        self.assertFalse(filter.matches_output('dir/main'))
        self.assertFalse(filter.matches_output('dir/dir/main.out'))
        self.assertTrue(filter.matches_output('dir/main.out'))

    def test_derives_set_output_path(self):
        filter = BaseFilter(output_path='dir/main.out')
        self.assertEqual(filter.derive_output_path('main.in'), 'dir/main.out')
        self.assertEqual(filter.derive_output_path('dir/main.in'), 'dir/main.out')

    def test_filters_a_single_input_file(self):
        filter = ReplaceFilter(pattern='Hello', replacement='World')
        path = self.mkfile('main.css', 'Hello')
        result = filter.filter(path)
        self.assertEqual('World', result)



from django.conf import settings

from assetfiles import assets
from assetfiles.filters import BaseFilter, ExtensionMixin
from assetfiles.tests.base import AssetfilesTestCase
import assetfiles.settings


class Filter1(ExtensionMixin, BaseFilter):
    input_exts = ('in', 'in1')
    output_ext = 'out'


class Filter2(ExtensionMixin, BaseFilter):
    input_ext = 'in2'
    output_ext = 'out2'


class TestAssets(AssetfilesTestCase):
    def setUp(self):
        super(TestAssets, self).setUp()
        self.old_filters = assetfiles.settings.FILTERS
        assetfiles.settings.FILTERS = (
            'assetfiles.tests.test_assets.Filter1',
            'assetfiles.tests.test_assets.Filter2',
        )

    def tearDown(self):
        assetfiles.settings.FILTERS = self.old_filters

    def test_find_asset(self):
        path1 = self.mkfile('static/some/dir/main.out.in')
        path2 = self.mkfile('static/another/dir/main.in2')
        asset_path1, filter1 = assets.find('some/dir/main.out')
        asset_path2, filter2 = assets.find('another/dir/main.out2')
        asset_path3, filter3 = assets.find('non/existent/file.out')
        self.assertEqual(path1, asset_path1)
        self.assertIsInstance(filter1, Filter1)
        self.assertEqual(path2, asset_path2)
        self.assertIsInstance(filter2, Filter2)
        self.assertEqual(None, asset_path3)
        self.assertEqual(None, filter3)



from django.template import loader, Context
from django.test.utils import override_settings
from django.utils import six

from assetfiles.tests.base import AssetfilesTestCase


class BaseTemplateTagsTestCase(AssetfilesTestCase):
    def render_template(self, template, **kwargs):
        if isinstance(template, six.string_types):
            template = loader.get_template_from_string(template)
        return template.render(Context(kwargs)).strip()

    def static_template_snippet(self, path, asvar=False):
        return "{%% load static from staticfiles %%}{%% static '%s' %%}" % path

    def assertStaticRenders(self, path, result, asvar=False, **kwargs):
        template = self.static_template_snippet(path, asvar)
        self.assertEqual(self.render_template(template, **kwargs), result)


class TestTemplateTag(BaseTemplateTagsTestCase):
    def test_template_tag(self):
        self.assertStaticRenders('does/not/exist.png',
                                 '/static/does/not/exist.png')
        self.assertStaticRenders('testfile.txt', '/static/testfile.txt')
        self.assertStaticRenders('testfile.txt', '/static/testfile.txt')


@override_settings(
    STATICFILES_STORAGE='django.contrib.staticfiles.storage.CachedStaticFilesStorage',
    DEBUG=False,
)
class TestStaticTagWithCachedStorage(BaseTemplateTagsTestCase):
    def test_raises_error_without_a_file(self):
        self.assertRaises(ValueError,
                          self.assertStaticRenders,
                          'does/not/exist.png',
                          '/static/does/not/exist.png')

    def test_does_not_change_folder_names(self):
        self.assertStaticRenders('path/',
                                 '/static/path/')
        self.assertStaticRenders('path/?query',
                                 '/static/path/?query')

    def test_returns_file_with_hash_name(self):
        self.mkfile('public/test/file.txt', 'Some Content')
        self.assertStaticRenders('test/file.txt',
                                 '/static/test/file.78138d2003f1.txt')
        self.assertStaticRenders('test/file.txt',
                                 '/static/test/file.78138d2003f1.txt')



from django.contrib.staticfiles import finders

from assetfiles import filters


def find(output_path):
    """
    Search for filters that would output the given file.

    If one is found, search for one the filter's possible input paths for
    the given output path. If there's a match return a tuple of the found
    absolute path and the found filter. Otherwise return tuple of (None, None).

    Args:
        output_path: An absolute path to search for.
    Returns
        A tuple containing the absolute path to the input file and the filter
        instance that would filter it.

    >>> find('/path/to/filtered.file')
    ('/path/to/filterable.file', <SomeFilter instance>)
    >>> find('/path/to/unfiltered.file')
    (None, None)
    """
    filter = filters.find_by_output_path(output_path)
    if filter:
        for input_path in filter.derive_input_paths(output_path):
            full_input_path = finders.find(input_path)
            if full_input_path:
                return full_input_path, filter
    return None, None



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import codecs
import os

from django.conf import settings
from django.contrib.staticfiles import storage
from django.core import management
from django.core.management.base import CommandError
from django.utils import six

from assetfiles.tests.base import AssetfilesTestCase


def call_command(*args, **kwargs):
    stdout = six.StringIO()
    kwargs.update({'stdout': stdout, 'stderr': six.StringIO(), 'verbosity': 1})
    management.call_command(*args, **kwargs)
    stdout.seek(0)
    return stdout


class TestFindStatic(AssetfilesTestCase):
    def setUp(self):
        super(TestFindStatic, self).setUp()
        self.mkfile('static/css/main.css')
        self.mkfile('app-1/static/css/main.css')
        self.mkfile('app-2/static/css/main.css')

    def test_find_file(self):
        out = call_command('findstatic', 'css/main.css', all=False)
        lines = out.readlines()[1:]
        self.assertEqual(len(lines), 1)
        self.assertIn('project/static/css/main.css', lines[0])

    def test_find_all_files(self):
        out = call_command('findstatic', 'css/main.css')
        lines = out.readlines()[1:]
        self.assertEqual(len(lines), 3)
        self.assertIn('project/static/css/main.css', lines[0])
        self.assertIn('app-1/static/css/main.css', lines[1])
        self.assertIn('app-2/static/css/main.css', lines[2])


class CustomStorage(storage.StaticFilesStorage):
    def post_process(self, paths, dry_run=False, **options):
        path_level = lambda name: len(name.split(os.sep))
        for name in sorted(paths.keys(), key=path_level, reverse=True):
            storage, path = paths[name]
            with storage.open(path) as original_file:
                new_name = name.replace('simple', 'complex')
                self._save(new_name, original_file)
                yield name, new_name, True


class TestCollectStatic(AssetfilesTestCase):
    def setUp(self):
        super(TestCollectStatic, self).setUp()
        self.old_staticfiles_storage = storage.staticfiles_storage

    def tearDown(self):
        super(TestCollectStatic, self).tearDown()
        storage.staticfiles_storage = self.old_staticfiles_storage

    def collectstatic(self, *args, **kwargs):
        kwargs.update({'interactive': False, 'clear': True})
        return call_command('collectstatic', *args, **kwargs)

    def assertStaticFileNotFound(self, path):
        static_path = os.path.join(settings.STATIC_ROOT, path)
        self.assertFalse(os.path.isfile(static_path),
            "file '{0}' exists".format(path))

    def assertStaticFileExists(self, path):
        static_path = os.path.join(settings.STATIC_ROOT, path)
        self.assertTrue(os.path.isfile(static_path),
            "file '{0}' does not exist".format(path))

    def assertStaticFileContains(self, path, text):
        static_path = os.path.join(settings.STATIC_ROOT, path)

        self.assertStaticFileExists(path)
        with codecs.open(static_path, 'r', encoding='utf-8') as file:
            file_data = file.read()
            self.assertIn(text, file_data,
                "'{0}' not in file '{1}'.".format(text, path))

    def test_copies_static_files(self):
        self.mkfile('static/css/static.css',
            'body { color: red; }')
        self.collectstatic()
        self.assertStaticFileContains('css/static.css',
            'body { color: red; }')

    def test_copies_unicode_static_files(self):
        self.mkfile('static/css/static.css',
            'a::before { content: "é"; }')
        self.collectstatic()
        self.assertStaticFileContains('css/static.css',
            'a::before { content: "é"; }')

    def test_copies_static_files_with_prefix(self):
        self.mkfile('static-prefix/css/static.css',
            'body { color: red; }')
        self.collectstatic()
        self.assertStaticFileContains('prefix/css/static.css',
            'body { color: red; }')

    def test_post_processes_static_files(self):
        storage.staticfiles_storage = CustomStorage()
        self.mkfile('static-prefix/css/simple.css',
            'body { color: red; }')
        self.collectstatic()
        self.assertStaticFileContains('prefix/css/complex.css',
            'body { color: red; }')

    def test_copies_app_static_files(self):
        self.mkfile('app-1/static/css/app_static.css',
            'body { color: blue; }')
        self.collectstatic()
        self.assertStaticFileContains('css/app_static.css',
            'body { color: blue; }')

    def test_processes_scss_files(self):
        self.mkfile('static/css/simple.scss',
            '$c: red; body { color: $c; }')
        self.collectstatic()
        self.assertStaticFileContains('css/simple.css',
            'body {\n  color: red; }')

    def test_processes_unicode_asset_files(self):
        self.mkfile('static/css/simple.scss',
            '$content: "é"; a::before { content: $content; }')
        self.mkfile('static/js/simple.coffee', 'a = foo: "é#{2}3"')
        self.collectstatic()
        self.assertStaticFileContains('css/simple.css', '@charset "UTF-8";')
        self.assertStaticFileContains('css/simple.css',
            'a::before {\n  content: "é"; }')
        self.assertStaticFileContains('js/simple.js', 'foo: "é" + 2 + "3"')

    def test_processes_scss_files_with_prefix(self):
        self.mkfile('static-prefix/css/simple.scss',
            '$c: red; body { color: $c; }')
        self.collectstatic()
        self.assertStaticFileContains('prefix/css/simple.css',
            'body {\n  color: red; }')

    def test_processes_app_scss_files(self):
        self.mkfile('app-1/static/css/app.scss',
            '$c: yellow; body { color: $c; }')
        self.collectstatic()
        self.assertStaticFileContains('css/app.css',
            'body {\n  color: yellow; }')

    def test_processes_scss_files_with_deps(self):
        self.mkfile('static/css/folder/_dep.scss', '$c: black;')
        self.mkfile('static/css/with_deps.scss',
            '@import "folder/dep"; body { color: $c; }')
        self.collectstatic()
        self.assertStaticFileContains('css/with_deps.css',
            'body {\n  color: black; }')

    def test_processes_scss_files_with_app_deps(self):
        self.mkfile('app-1/static/css/folder/_dep.scss', '$c: white;')
        self.mkfile('static/css/with_app_deps.scss',
            '@import "folder/dep"; body { color: $c; }')
        self.collectstatic()
        self.assertStaticFileContains('css/with_app_deps.css',
            'body {\n  color: white; }')

    def test_skips_sass_dependencies(self):
        self.mkfile('static/css/_dep.scss', '$c: black;')
        self.mkfile('static/css/with_deps.scss',
            '@import "dep"; body { color: $c; }')
        self.collectstatic()
        self.assertStaticFileNotFound('css/_dep.css')
        self.assertStaticFileNotFound('css/_dep.scss')

    def test_processes_coffee_files(self):
        self.mkfile('static/js/simple.coffee', 'a = foo: "1#{2}3"')
        self.collectstatic()
        self.assertStaticFileContains('js/simple.js', 'foo: "1" + 2 + "3"')

    def test_post_processes_asset_files(self):
        storage.staticfiles_storage = CustomStorage()
        self.mkfile('static/css/simple.scss',
            '$c: red; body { color: $c; }')
        self.collectstatic()
        self.assertStaticFileContains('css/complex.css',
            'body {\n  color: red; }')

    def test_post_processes_asset_files_with_prefix(self):
        storage.staticfiles_storage = CustomStorage()
        self.mkfile('static-prefix/css/simple.scss',
            '$c: red; body { color: $c; }')
        self.collectstatic()
        self.assertStaticFileContains('prefix/css/complex.css',
            'body {\n  color: red; }')

    def test_does_not_allow_symlinking(self):
        error = CommandError if six.PY3 else SystemExit
        with self.assertRaises(error):
            self.collectstatic(link=True)



class BaseFilter(object):
    """
    Base class for filters that process files.
    """
    # input_path = None
    # output_path = None

    def __init__(self, input_path=None, output_path=None):
        self.input_path = input_path
        self.output_path = output_path

    def matches_input(self, input_path):
        """
        Returns true if the input file should be processed by this filter.

        Args:
            input_path: A file path, relative to the static dir.
        """
        if self.input_path:
            return input_path is self.input_path
        else:
            return self._matches_input(input_path)

    def _matches_input(self, input_path):
        return False

    def matches_output(self, output_path):
        """
        Returns true if the output file would be created by this filter.

        Args:
            output_path: A file path, relative to the static dir.
        """
        if self.output_path:
            return output_path is self.output_path
        else:
            return self._matches_output(output_path)

    def _matches_output(self, input_path):
        return False

    def is_filterable(self, output_path):
        """
        Determines wether to filter the file with the given output path.

        Implement this method if there are certain paths that should not
        be processed by this filter, i.e. they are dependencies.

        Args:
            output_path: A file path, relative to the static dir.
        """
        return True

    def derive_input_paths(self, output_path):
        """Returns possible input paths that would create the given output path.

        Args:
            output_path: A file path, relative to the static dir.
        Returns
            A list of search paths. Can be empty.
        """
        if self.input_path:
            return [self.input_path]
        else:
            return self._derive_input_paths(output_path)

    def _derive_input_paths(self, output_path):
        return []

    def derive_output_path(self, input_path):
        """
        Returns the path to the ouput file created with the given input path.

        Args:
            input_path: A file path, relative to the static dir.
        Returns:
            An path to the output file, relative to the static dir.
        """
        if self.output_path:
            return self.output_path
        else:
            return self._derive_output_path(input_path)

    def _derive_output_path(self, input_path):
        return None

    def filter(self, input_path):
        """
        Filters the file with the given input path.

        All filters must implement this method.

        Args:
            input_path: An absolute path to the file to filter.

        Returns:
            The filtered string.
        """
        raise NotImplementedError()



import os

from django.contrib.staticfiles.management.commands import collectstatic
from django.contrib.staticfiles import finders
from django.core.management.base import CommandError
from django.utils.datastructures import SortedDict

from assetfiles import filters
from assetfiles.storage import TempFilesStorage


class Command(collectstatic.Command):
    """
    Overrides staticfiles' `collectstatic` command to filter files before
    copying them to the target storage.
    """
    def __init__(self, *args, **kwargs):
        super(Command, self).__init__(*args, **kwargs)
        self.found_files = SortedDict()
        self.temp_storage = TempFilesStorage()

    def set_options(self, **options):
        super(Command, self).set_options(**options)
        if self.symlink:
            raise CommandError('Symlinking is not supported by Assetfiles.')

    def collect(self):
        if self.clear:
            self.clear_dir('')
        self._collect_files()
        if self.post_process and hasattr(self.storage, 'post_process'):
            self._post_process_files()

        return {
            'modified': self.copied_files,
            'unmodified': self.unmodified_files,
            'post_processed': self.post_processed_files,
        }

    def _collect_files(self):
        for path, prefixed_path, source_storage in self._full_file_list():
            if self._is_copied(prefixed_path):
                self.log("Skipping '%s' (already copied earlier)" % path)
            elif self.delete_file(path, prefixed_path, source_storage):
                path, prefixed_path, source_storage = self._collect_file(
                    path, prefixed_path, source_storage)
            self.found_files[prefixed_path] = (source_storage, path)

    def _post_process_files(self):
        processor = self.storage.post_process(self.found_files,
                                              dry_run=self.dry_run)
        for original_path, processed_path, processed in processor:
            if processed:
                self.log("Post-processed '%s' as '%s" %
                         (original_path, processed_path), level=1)
                self.post_processed_files.append(original_path)
            else:
                self.log("Skipped post-processing '%s'" % original_path)

    def _collect_file(self, path, prefixed_path, source_storage):
        source_path = source_storage.path(path)
        target_path = prefixed_path
        filter = filters.find_by_input_path(prefixed_path)

        if filter and not filter.is_filterable(prefixed_path):
            self.log("Skipping '%s' (filter dependency)" % path)
        else:
            if filter:
                target_path, source_storage = self._filter_file(
                    filter, prefixed_path, source_path, source_storage)
            self._copy_file(source_path, target_path, source_storage)
            if not prefixed_path in self.copied_files:
                self.copied_files.append(prefixed_path)

        return (source_path, target_path, source_storage)

    def _filter_file(self, filter, prefixed_path, source_path, source_storage):
        target_path = filter.derive_output_path(prefixed_path)
        if self.dry_run:
            self.log("Pretending to process '%s'" % source_path, level=1)
        else:
            self.log("Processing '%s'" % source_path, level=1)
            content = filter.filter(source_path)
            source_storage = self.temp_storage
            source_storage.save(source_path, content)

        return (target_path, source_storage)

    def _copy_file(self, source_path, target_path, source_storage):
        if self.dry_run:
            self.log("Pretending to copy '%s'" % source_path, level=1)
        else:
            self.log("Copying '%s'" % source_path, level=1)
            with source_storage.open(source_path) as source_file:
                self._make_local_dirs(target_path)
                self.storage.save(target_path, source_file)

    def _full_file_list(self):
        for finder in finders.get_finders():
            for path, source_storage in finder.list(self.ignore_patterns):
                prefixed_path = self._get_prefixed_path(path, source_storage)
                yield (path, prefixed_path, source_storage)

    def _get_prefixed_path(self, path, source_storage):
        if getattr(source_storage, 'prefix', None):
            prefixed_path = os.path.join(source_storage.prefix, path)
        else:
            prefixed_path = path
        return prefixed_path

    def _make_local_dirs(self, target_path):
        if self.local:
            full_path = self.storage.path(target_path)
            try:
                os.makedirs(os.path.dirname(full_path))
            except OSError:
                pass

    def _is_copied(self, file):
        return file in self.found_files or file in self.copied_files



"""
Views and functions for serving assets and static files. These are only to be
used during development, and SHOULD NOT be used in a production setting.
"""
import mimetypes
import os
import posixpath
try:
    from urllib.parse import unquote
except ImportError:     # Python 2
    from urllib import unquote

from django.conf import settings
from django.contrib.staticfiles import finders
from django.core.exceptions import ImproperlyConfigured
from django.http import Http404, HttpResponse
from django.views import static

from assetfiles import assets


def serve(request, path, document_root=None, insecure=False, **kwargs):
    """
    Serve static files below a given point in the directory structure or
    from locations inferred from the staticfiles finders.

    To use, put a URL pattern such as::

        (r'^(?P<path>.*)$', 'assetfiles.views.serve')

    in your URLconf.

    It uses the django.views.static view to serve the found files.
    """
    if not settings.DEBUG and not insecure:
        raise ImproperlyConfigured('The staticfiles view can only be used in '
                                   'debug mode or if the the --insecure '
                                   "option of 'runserver' is used")
    normalized_path = posixpath.normpath(unquote(path)).lstrip('/')

    static_path = finders.find(normalized_path)
    if static_path:
        document_root, path = os.path.split(static_path)
        return static.serve(request, path, document_root=document_root, **kwargs)

    asset_path, filter = assets.find(normalized_path)
    if asset_path:
        content = filter.filter(asset_path)
        mimetype, encoding = mimetypes.guess_type(normalized_path)
        return HttpResponse(content, content_type=mimetype)

    if path.endswith('/') or path == '':
        raise Http404('Directory indexes are not allowed here.')
    raise Http404("'%s' could not be found" % path)



from django.conf import settings


FILTERS = getattr(settings, 'ASSETFILES_FILTERS', (
    'assetfiles.filters.sass.SassFilter',
    'assetfiles.filters.coffee.CoffeeScriptFilter'
))

SASS_DIRS = getattr(settings, 'ASSETFILES_SASS_DIRS', ('css',))



from __future__ import unicode_literals

import os
import pipes
from subprocess import Popen, PIPE

from django.conf import settings
from django.contrib.staticfiles.finders import find
from django.utils.encoding import smart_text

from assetfiles.filters import BaseFilter, ExtensionMixin
import assetfiles.settings


class SassFilter(ExtensionMixin, BaseFilter):
    """
    Filters Sass files into CSS.

    Attributes:
        functions_path: A path to the Sass extension functions for Django
            integration.
    """
    input_exts = ('sass', 'scss')
    output_ext = 'css'
    functions_path = os.path.abspath(
        os.path.join(os.path.dirname(__file__), '../scripts/sass_functions.rb'))

    def __init__(self, options=None, *args, **kwargs):
        super(SassFilter, self).__init__(*args, **kwargs)
        if options is None:
            options = {}
        if 'compass' not in options:
            options['compass'] = self._detect_compass()
        self.options = options

    def filter(self, input):
        command = 'sass {0} {1}'.format(
            ' '.join(self._build_args()),
            pipes.quote(input))

        env = dict(os.environ)
        env.update({'DJANGO_STATIC_URL': settings.STATIC_URL})

        process = Popen(command, shell=True, stdout=PIPE, stderr=PIPE, env=env)
        stdout, stderr = process.communicate()

        if process.returncode:
            raise SassFilterError(stderr)
        else:
            return stdout

    def is_filterable(self, output_path):
        """
        Skips files prefixed with a '_'. These are Sass dependencies.
        """
        _, file_name = os.path.split(output_path)
        return not file_name.startswith('_')

    def _build_args(self):
        """
        Returns a list of arguments for the Sass command.
        """
        args = []
        if self.options['compass']:
            args.append('--compass')
        if self.functions_path is not None:
            args.append(
                '--require {0}'.format(pipes.quote(self.functions_path)))
        if sass_load_paths:
            for path in sass_load_paths:
                args.append('--load-path {0}'.format(pipes.quote(path)))
        return args

    def _detect_compass(self):
        """
        Returns true if Compass integration is available.
        """
        return os.system('which compass > /dev/null') is 0


def get_static_sass_dirs(dirs=None):
    """
    Returns the directories with Sass files within the static directories.

    Args:
        dirs: A list or tuple of directory names that contain Sass files.
            Can be configured with the ASSETFILES_SASS_DIRS setting, which by
            default is `('css',)`

    Returns:
        A list of directory paths containing Sass files.
    """
    if not dirs:
        dirs = assetfiles.settings.SASS_DIRS

    load_paths = []
    for dir in dirs:
        load_paths += find(dir, all=True) or []
    return load_paths


"""
Directories that will be added to the Sass load path.

By default, these are 'css' directories within the static directories.
"""
sass_load_paths = get_static_sass_dirs()


class SassFilterError(Exception):
    """
    SassError parses syntax errors generated by Sass and formats them
    for Django.
    """
    def __init__(self, stacktrace):
        stacktrace = stacktrace.strip().split(b'\n')[:-1]
        message = b'\n'.join(stacktrace)
        super(SassFilterError, self).__init__(message)



from django.contrib.staticfiles.handlers import StaticFilesHandler

from assetfiles.views import serve


class AssetFilesHandler(StaticFilesHandler):
    """
    Overrides StaticFilesHandler to use Assetfiles' view for serving files.
    """

    def serve(self, request):
        return serve(request, self.file_path(request.path), insecure=True)



import os
import shutil
import tempfile

from django.conf import settings
from django.contrib.staticfiles import finders, storage
from django.test import TestCase
from django.utils import six
from django.utils.functional import empty

from assetfiles import assets, filters
import assetfiles.settings


def filter(path):
    asset_path, filter = assets.find(path)
    return filter.filter(asset_path).strip()


def assertRaisesRegex(self, *args, **kwargs):
    attr = 'assertRaisesRegex' if six.PY3 else 'assertRaisesRegexp'
    return getattr(self, attr)(*args, **kwargs)


def is_glob2_available():
    try:
        import glob2
        return True
    except ImportError:
        return False


class AssetfilesTestCase(TestCase):
    def setUp(self):
        # Clear the cached staticfiles_storage out, this is because when it first
        # gets accessed (by some other test), it evaluates settings.STATIC_ROOT,
        # since we're planning on changing that we need to clear out the cache.
        storage.staticfiles_storage._wrapped = empty
        # Clear the cached staticfile finders, so they are reinitialized every
        # run and pick up changes in settings.STATICFILES_DIRS.
        finders._finders.clear()
        # Clear the cached assetfile filters, so they are reinitialized every
        # run and pick up changes in settings.ASSETFILES_FILTERS.
        filters._filters.clear()

        if not os.path.exists(settings.PROJECT_ROOT):
            shutil.copytree(
                os.path.join(settings.TESTS_ROOT, 'project-template'),
                settings.PROJECT_ROOT
            )
        self.addCleanup(shutil.rmtree, settings.PROJECT_ROOT, ignore_errors=True)
        self.root = settings.PROJECT_ROOT

    def mkfile(self, path, content=None):
        abspath = os.path.join(self.root, path)
        dirname = os.path.dirname(abspath)

        if not os.path.isdir(dirname):
            os.makedirs(dirname)

        with open(abspath, 'w') as file:
            if content:
                if not six.PY3:
                    content = content.encode('utf-8')
                file.write(content)

        return abspath



import pipes
from subprocess import Popen, PIPE

from django.utils.encoding import smart_text

from assetfiles.filters import BaseFilter, ExtensionMixin


class CoffeeScriptFilter(ExtensionMixin, BaseFilter):
    """
    Filters CoffeeScript files into JS.
    """
    input_ext = 'coffee'
    output_ext = 'js'

    def filter(self, input):
        command = 'coffee --compile --print {0}'.format(
            pipes.quote(input))

        process = Popen(command, shell=True, stdout=PIPE, stderr=PIPE)
        stdout, stderr = process.communicate()

        if process.returncode:
            raise CoffeeScriptFilterError(stderr)
        else:
            return stdout


class CoffeeScriptFilterError(Exception):
    """
    CoffeeScriptError parses syntax errors generated by CoffeeScript and
    formats them for Django.
    """
    pass



from setuptools import find_packages, setup

from assetfiles import __version__

setup(
    name='django-assetfiles',
    version=__version__,
    description='Drop-in replacement for staticfiles which handles asset processing.',
    author='LocalMed',
    author_email='pete.browne@localmed.com',
    url='http://github.com/LocalMed/django-assetfiles',
    license='MIT',
    packages=find_packages(),
    include_package_data=True,
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Topic :: Internet :: WWW/HTTP',
    ],
)



from __future__ import unicode_literals

from django.test import TestCase

from assetfiles.storage import TempFilesStorage


class TestTempFilesStorage(TestCase):
    def test_stores_in_memory(self):
        storage = TempFilesStorage()
        storage.save('path/to/file.txt', 'Hello World!')
        self.assertTrue(storage.exists('path/to/file.txt'))

    def test_opens_strings_as_content_files(self):
        storage = TempFilesStorage()
        storage.save('path/to/file.txt', 'Hello World!')
        file = storage.open('path/to/file.txt')
        self.assertEqual(u'Hello World!', u''.join(file.chunks()))

    def test_deletes_storage(self):
        storage = TempFilesStorage()
        storage.save('path/to/file.txt', 'Hello World!')
        storage.delete('path/to/file.txt')
        self.assertFalse(storage.exists('path/to/file.txt'))
